@@ -1,6 +1,6 @@
 #! /bin/sh
 # check script for Lzd - Educational decompressor for lzip files
-# Copyright (C) 2013-2015 Antonio Diaz Diaz.
+# Copyright (C) 2013-2016 Antonio Diaz Diaz.
 #
 # This script is free software: you have unlimited permission
 # to copy, distribute and modify it.
@@ -19,7 +19,7 @@ fi
 
 if [ -d tmp ] ; then rm -rf tmp ; fi
 mkdir tmp
-cd "${objdir}"/tmp
+cd "${objdir}"/tmp || framework_failure
 
 in="${testdir}"/test.txt
 in_lz="${testdir}"/test.txt.lz
